from typing import *
from twopilabs.utils.scpi import *
import datetime


class ScpiSystem(object):
    """Class containing SCPI commands concerning SYSTEM subsystem"""
    def __init__(self, device: ScpiDevice) -> None:
        self.device = device

    def error_next(self) -> ScpiEvent:
        return self.device.execute('SYST:ERR:NEXT?', result=ScpiEvent)

    def version(self) -> str:
        """returns supported SCPI version"""
        version = self.device.execute('SYST:VERSION?', result=ScpiChars).as_string()

        self.device.raise_error()
        return version

    def preset(self) -> None:
        """Loads the system defaults"""
        self.device.execute('SYST:PRESET')

    def time(self, time: Optional[datetime.time] = None) -> datetime.time:
        """Sets or gets the radar system time"""
        if time is not None:
            self.device.execute('SYST:TIME', param=ScpiNumberArray([time.hour, time.minute, time.second]))
        else:
            time = datetime.time(*self.device.execute('SYST:TIME?', result=ScpiNumberArray).as_int_list())

        self.device.raise_error()
        return time

    def date(self, date: Optional[datetime.date] = None) -> datetime.date:
        """Sets or gets the radar system date"""
        if date is not None:
            self.device.execute('SYST:DATE', param=ScpiNumberArray([date.year, date.month, date.day]))
        else:
            date = datetime.date(*self.device.execute('SYST:DATE?', result=ScpiNumberArray).as_int_list())

        self.device.raise_error()
        return date

    def timezone(self, posix: Optional[str] = None) -> str:
        """Sets or gets the local timezone in POSIX1003.1 format"""
        if posix is not None:
            self.device.execute('SYST:TZONE', param=ScpiString(posix))
        else:
            posix = self.device.execute('SYST:TZONE?', result=ScpiString).as_string()

        self.device.raise_error()
        return posix

    def utc(self, timestamp: Optional[float] = None):
        """Sets or gets the radar system clock as UTC POSIX timestamp"""
        if timestamp is not None:
            self.device.execute('SYST:UTC', param=ScpiNumber(timestamp))
        else:
            timestamp = self.device.execute('SYST:UTC?', result=ScpiNumber).as_float()

        self.device.raise_error()
        return timestamp


from typing import *
from twopilabs.utils.scpi import *


class ScpiCore(object):
    """Class containing core SCPI commands"""

    def __init__(self, device: ScpiDevice) -> None:
        self.device = device

    def idn(self) -> List[str]:
        """Executes a \*IDN? command to read device identification string"""
        idn = self.device.execute('*IDN?', result=ScpiChars)
        return idn.as_string().split(',')

    def cls(self) -> None:
        """Executes a \*CLS command to clear all status data structures in the device"""
        self.device.execute('*CLS')

    def rst(self) -> None:
        """Executes a \*RST command to stop execution of all overlapped (asynchronous) commands and resets the device"""
        self.device.execute('*RST')

    def sav(self, slot: int = 0 ) -> None:
        """Executes a \*SAV command to save system state into given slot

        :param slot: Memory register slot
        """
        self.device.execute('*SAV', param=ScpiNumber(int(slot)))

    def rcl(self, slot: int = 0) -> None:
        """Executes a \*RCL command to recall system state from given slot

        :param slot: Memory register slot
        """
        self.device.execute('*RCL', param=ScpiNumber(int(slot)))

    def sds(self, slot: int = 0) -> None:
        """Executes a \*SDS command to set the system state in given slot to defaults

        :param slot: Memory register slot
        """
        self.device.execute('*SDS', param=ScpiNumber(int(slot)))

    def trg(self) -> None:
        """Executes a \*TRG command to soft-trigger the device in initiated state waiting for a trigger"""
        self.device.execute('*TRG')

    def wai(self) -> None:
        """Executes a \*WAI command to wait for all overlapped (asynchronous) commands to finish"""
        self.device.execute('*WAI')

    def opc(self) -> bool:
        """Executes a \*OPC? command to wait for all overlapped (asynchronous) commands to finish and returns '1'"""
        completed = self.device.execute('*OPC?', result=ScpiBool)
        return completed.as_bool()



import io
from typing import *
from twopilabs.utils.scpi import *
from .acquisition import *


class ScpiCalc(object):
    """Class containing SCPI commands concerning CALCULATE subsystem"""

    def __init__(self, device: ScpiDevice):
        self.device = device

    def trace_list(self, traces: Optional[List[int]] = None) -> List[int]:
        """Sets or gets the list of selected traces that are processed and for which CALC data is available"""
        if traces is not None:
            self.device.execute('CALCULATE:TRACE:LIST', param=ScpiNumList(traces))
        else:
            traces = self.device.execute('CALCULATE:TRACE:LIST?', result=ScpiNumList).as_list()

        self.device.raise_error()
        return traces

    def data(self, wait: bool=True) -> Optional[Acquisition]:
        # Only reads the header information and returns a stream for further reading of data
        stream = self.device.execute('*WAI;CALC:DATA?' if wait else 'CALC:DATA?', result=ScpiArbStream)
        if stream is None or stream._length == 0:
            # Radar has no data in FIFO
            self.device.raise_error()
            return None
        return Acquisition._from_stream(stream)

    def data_all(self, acq_count: int=None, wait: bool=True) -> List[Acquisition]:
        """Reads all (or given number of) acquisitions from radar and returns.
           Optionally waits for data to be available initially"""
        memory_stream = self.device.execute(f'{"*WAI;" if wait else ""}CALC:DATA:ALL?',
                                            param=ScpiNumber(acq_count) if acq_count is not None else None,
                                            result=ScpiArbBlock)
        acquisitions = []

        if memory_stream is None or memory_stream._length == 0:
            # Radar has no data in FIFO
            self.device.raise_error()
            return acquisitions

        while memory_stream._remaining > 0:
            # Create acquisition object from the in-memory stream and store into list object
            acq = Acquisition._from_stream(ScpiArbBlock(memory_stream.as_memoryview()))
            acquisitions.append(acq)
            memory_stream.seek(acq.header.header_length +
                               acq.header.sweep_count * acq.header.trace_count * acq.header.trace_size,
                               io.SEEK_CUR)

        return acquisitions

    def data_available(self, wait: bool=True) -> List[Acquisition]:
        """Reads all (or given number of) acquisitions from radar and returns.
           Optionally waits for data to be available initially"""
        memory_stream = self.device.execute(f'{"*WAI;" if wait else ""}CALC:DATA:AVAILABLE?',
                                            result=ScpiArbBlock)
        acquisitions = []

        if memory_stream is None or memory_stream._length == 0:
            # Radar has no data in FIFO
            self.device.raise_error()
            return acquisitions

        while memory_stream._remaining > 0:
            # Create acquisition object from the in-memory stream and store into list object
            acq = Acquisition._from_stream(ScpiArbBlock(memory_stream.as_memoryview()))
            acquisitions.append(acq)
            memory_stream.seek(acq.header.header_length +
                               acq.header.sweep_count * acq.header.trace_count * acq.header.trace_size,
                               io.SEEK_CUR)

        return acquisitions


from typing import *
from twopilabs.utils.scpi import *
from .acquisition import *

class ScpiInitiate(object):
    """Class containing SCPI commands concerning the INITIATE subsystem"""

    def __init__(self, device: ScpiDevice) -> None:
        self.device = device

    def immediate(self):
        """Immediately initiates (arms) the device for a single trigger event.
        When trigger source is IMMEDIATE (default), the device will trigger immediately."""
        self.device.execute("INIT:IMM")
        self.device.raise_error()

    def immediate_and_receive(self) -> Optional[Acquisition]:
        """Immediately initiates (arms) the device, waits for trigger and returns an acquisition object."""
        self.immediate()
        return self.device.calc.data(wait=True)

    def continuous(self, enabled: Optional[bool]=None) -> bool:
        """Enters or exits the continuous initiate mode

        :param enabled: Continuous mode enabled or disabled
        """
        if enabled is not None:
            self.device.execute('INIT:CONT', param=ScpiBool(enabled))
        else:
            enabled = self.device.execute('INIT:CONT?', result=ScpiBool).as_bool()

        self.device.raise_error()
        return enabled